
/* Terminality - a portable terminal handling library
 * Copyright (C) 1999-2001, Emil Mikulic.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Linked list code
 * Written by Emil Mikulic, 1999.
 * Overhauled by Michal Safranek, 2001.
 *
 * $Id: list.h,v 1.7 2001/09/05 02:28:51 darkmoon Exp $
 */

#ifndef __LIST_H__
#define __LIST_H__

#include "xmem.h"
#include <stdio.h>
#include <string.h>

/*
 * Short note about complexity:
 * This list is primarily intended to hold medium count of items and provide
 * fast access to them. Thus using list_delete() results in performance
 * disaster (for large lists) because it uses memmove. The lower element
 * you remove the larger block of memory must be moved.
 * (it moves n+1 to count items [* 4 bytes (sizeof ptr)])
 *
 * This means one thing: using this list as 'engine' for 'stack' is
 * not-so-bad idea but using it for 'queue' is really _bad_ idea.
 * ==========================================================================
 * Even worse: when using list_delete the memory used by list is not
 * reallocated to make the list smaller and thus making list with 1M items
 * and then using list_delete won't free *any* memory. I won't guarantee
 * this will change in future ... feel free to hack this in if you want to.
 */


#ifdef __cplusplus
extern "C" {
#endif



#define STARTING_POOL 5

/* list */
typedef struct {
	/* Don't change these or you'll get into serious troubles */
	void **elements;
	int count;
	int allocated;
	/* Expecting many list_add()s? Increase alloc_by to make insertion more
	 * effective. Warning: large values results in memory exhaustion!
	 * count == alloc_by + 1 elements results in
	 * xmalloc(2 * alloc_by * sizeof(void *)); ! Be careful.
	 */
	int alloc_by;
} tn_list;



/* create new list element */
tn_list *tn_list_new(void *x);

/* add an element on to a list */
int tn_list_add(tn_list *dest, void *elem);

/* get the number of items in the list */
int tn_list_size(tn_list *x);

/* free the whole list */
void tn_list_free(tn_list *x);

/* get the nth item (data) in a list */
void *tn_list_getdata(tn_list *x, int n);

/* delete the nth element of list */
int tn_list_delete(tn_list *x, int n);

/* replace nth element in a list */
void tn_list_replace(tn_list *dest, int n, void *elem);

/* free space used by list */
void tn_list_kill(tn_list *x);

#ifdef __cplusplus
}
#endif

#endif



/*
 * $Log: list.h,v $
 * Revision 1.7  2001/09/05 02:28:51  darkmoon
 * (Michal) Added usage hints
 * (Michal) Added tn_list_replace()
 *
 * Revision 1.6  2001/02/18 05:50:18  darkmoon
 * LGPL'd
 *
 * Revision 1.5  2001/02/16 02:41:40  darkmoon
 * Changed all instances of the list to tn_list
 *
 * Revision 1.4  2001/01/05 15:14:54  darkmoon
 * (Michal) Overhauled linked list code
 * It's not really a linked list anymore, more like a flat lookup
 *
 * Revision 1.3  2000/09/04 07:36:17  darkmoon
 * Polished up for Terminality 1.4 release
 *
 * Revision 1.2  1999/11/22 09:30:12  darkmoon
 * Added short disclaimer and copyright notice to beginning of file.
 *
 * Revision 1.1  1999/11/20 10:12:39  darkmoon
 * Already developed linked-list code pulled into Terminality
 *
 * Revision 1.3  1999/11/08 00:51:08  darkmoon
 * Moved revision log to end of file.
 *
 * Revision 1.2  1999/11/06 06:16:17  darkmoon
 * Added rcsid[]s, tweaked headers, added 'extern "C" {' for C++ compilers
 *
 * Revision 1.1.1.1  1999/11/06 05:04:39  darkmoon
 * Initial revision
 *
 * Revision 1.1  1999/10/26 11:40:39  darkmoon
 * Major restructuring, pulling in some extra files.
 *
 * Revision 1.2  1999/10/26 02:50:14  darkmoon
 * Redid headers to use CVS ID and LOG tags
 *
 */

