
/* Terminality - a portable terminal handling library
 * Copyright (C) 1999-2001, Emil Mikulic.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: tn.h,v 1.24 2001/10/19 13:42:49 darkmoon Exp $ */

/* Project:     Terminality
 * File:	tn.h
 * Author:      Emil Mikulic
 * Description: Provide a general interface to the Terminality library
 */

#ifndef _tn_h_
#define _tn_h_

/* Version-specifics */
#define TN_VERSION "2.0"
#define TN_BUILD 20

#include "colors.h"

/* Platform-dependent includes */
#ifdef USE_NCURSES
#       include "tn_nc.h"
#	undef _WIN32
#endif
#ifdef _WIN32
#	include "tn_w32.h"
#endif

/* Safeguard */
#ifndef TN_H_IMPLEMENTED
#	error Unrecognised platform: -DUSE_NCURSES on Unix platforms
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* Colors */
typedef enum {
	Default = -1, /* Not a real color, needed for listing */
	Black = 0,
	Blue = 1,
	Green = 2,
	Cyan = 3,
	Red = 4,
	Magenta = 5,
	Brown = 6,
	LightGray = 7,
	/* Bright [foreground] */
	DarkGray = 8,
	LightBlue = 9,
	LightGreen = 10,
	LightCyan = 11,
	LightRed = 12,
	LightMagenta = 13,
	Yellow = 14,
	White = 15 
} color;

#define LightGrey LightGray
#define DarkGrey DarkGray
#define LightBrown Yellow

#define colour color
#define COLOUR COLOR

/* Cursor struct */
typedef enum {
    none,	/* hidden */
    line,	/* thin */
    rect	/* fat */
} cursor;



/* Functions */
void initcons(void);			/* Initialise console */
void donecons(void);			/* Done with console */
void clrscr(void);			/* Clear screen */
void textcolor(color c);		/* Set foreground (text) color */
void textbackground(color c);		/* Set background color */
void setcolor(color f, color b);	/* Set color to f, b */
color getbgcolor(void);			/* Get bg color */
color getfgcolor(void);			/* Get fg color */
key readkey(void);			/* Get keypress */
int keypressed(void);			/* Key pressed? */
#ifndef __DJGPP__
void delay(int ms);			/* Delay for <ms> milliseconds */
#endif
void gotoxy(int x, int y);		/* Move cursor to x,y */
void writech(chtype c);			/* Write character */
int update_set(int want);		/* Enable / disable screen updates */
void update(void);			/* Update screen */
int has_color(void);			/* Do we have colors? */
void clreol(void); 			/* Clear to end of line */
int wherey(void);			/* Return cursor Y position */
int wherex(void);			/* Return cursor X position */
int beep(void);				/* Audible beep/bell/alarm thing */
int printw(const char *format, ...);	/* printf to screen */

/* No-color terminal functions */
void highvideo(void);			/* Bright text */
void lowvideo(void);			/* Dim text */
void normvideo(void);			/* Normal text */
void reversevideo(void);		/* Reverse/standout text */

/* Cursor functions */
cursor get_cursor(void);		/* Gets cursor */
void set_cursor(cursor c);		/* Sets cursor */

/* Color scheme functions: */
color get_scheme_color(int name);	/* Returns color by constant */
void set_scheme(color *scheme);		/* Sets color scheme */
void default_scheme(void);		/* Sets default color scheme */



/* Compatibility with older Terminality releases */
#define fgcolor textcolor
#define bgcolor textbackground

/* COLOR attribute array */
extern int COLOR[16][8];

/* Global cursor */
extern cursor glb_cursor;

/* The following special characters must be supported or emulated:
 *	HLINE	- Horizontal line
 *	VLINE	- Vertical line
 *	CROSS	- Cross/plus
 *	C_UL	- Upper left corner
 *	C_UR	- Upper right
 *	C_LL	- Lower left
 *	C_LR	- Lower right
 *	BLOCK_1	- 25%-full block
 *	BLOCK_2	- 50%-full block
 *	BLOCK_3 - 75%-full block
 *	BLOCK_4	- Solid block
 *	TEE_L	- Tee pointing left
 *	TEE_R	- Tee pointing right
 *	TEE_U	- Tee pointing up
 *	TEE_D	- Tee pointing down
 *	ARROW_L	- Arrow pointing left
 *	ARROW_R	- Arrow pointing right
 *	ARROW_U	- Arrow pointing up
 *	ARROW_D	- Arrow pointing down
 *
 *
 * The macro _c(x) should be provided. It is used to create a
 * chtype value from an integer specifying a regular ASCII
 * character code. i.e. BLOCK_4 = _c(219)
 *
 *
 * The following keys must be provided:
 *	KEY_ENTER
 *	KEY_TAB
 *	KEY_BACKSPACE
 *	KEY_DEL
 *	KEY_UP
 *	KEY_DOWN
 *	KEY_LEFT
 *	KEY_RIGHT
 *	KEY_HOME
 *	KEY_END
 *	KEY_PGUP
 *	KEY_PGDOWN
 */

/* To obtain the size of the terminal, Terminality backends must
   provide CON_COLS and CON_ROWS as macros or variables. */



/* Types */
typedef enum {
	Unknown = 0,
	Checkbox = 1,
	Radio = 2,
	Textbox = 3,
	Textarea = 4,
	Button = 5,
	Menu = 6,
	Hmenu = 7,
	Listing = 8,
	Form = 9,
	Custom = 10, /* Custom type for drawing graphics that are persistant
					even when redraw is called */
	Textview = 11,
} element_type;



#ifdef __cplusplus
}
#endif

#endif


/*
 * $Log: tn.h,v $
 * Revision 1.24  2001/10/19 13:42:49  darkmoon
 * Minor cosmetic fixes
 *
 * Revision 1.23  2001/10/19 13:33:18  darkmoon
 * The big two point oh!
 * Moved extern "C" wrapper so as not to interfere with BCC's stdlib.h
 *
 * Revision 1.22  2001/09/05 02:31:05  darkmoon
 * (Michal) Added custom and Textview element types
 * (Michal) Added get[fb]gcolor()
 * Version upped to 1.9 for release
 *
 * Revision 1.21  2001/04/23 10:57:17  darkmoon
 * Updated for v1.8
 *
 * Revision 1.20  2001/04/02 11:31:03  darkmoon
 * Changed a C++ comment into a C comment for stupider C-only compilers
 * Indented a few comments
 * Upped the build numbers
 *
 * Revision 1.19  2001/02/20 11:23:04  darkmoon
 * Minor cosmetic changes
 *
 * Revision 1.18  2001/02/18 05:50:18  darkmoon
 * LGPL'd
 *
 * Revision 1.17  2001/01/07 09:54:32  darkmoon
 * Fixed up cursor and color scheme function definitions
 *
 * Revision 1.16  2001/01/05 17:16:30  darkmoon
 * Merged in Michal's changes (lots of)
 *
 * Revision 1.15  2000/09/09 08:25:03  darkmoon
 * Welcome to Terminality 1.5
 *
 * Revision 1.14  2000/09/04 07:36:17  darkmoon
 * Polished up for Terminality 1.4 release
 *
 * Revision 1.13  2000/09/04 06:38:01  darkmoon
 * (Michal) Fixed DJGPP typo
 *
 * Revision 1.12  2000/08/06 07:16:44  darkmoon
 * Updated version to 1.3
 * Made the Unrecognised Platform error a bit more helpful
 * Ported to work with DJGPP/pdcurses
 *
 * Revision 1.11  2000/04/11 06:12:52  darkmoon
 * Updated version/build info
 *
 * Revision 1.10  1999/11/22 22:25:34  darkmoon
 * printw now takes const char *format, not just char *format
 *
 * Revision 1.9  1999/11/22 11:07:42  darkmoon
 * Updated to v1.0
 *
 * Revision 1.8  1999/11/22 09:30:12  darkmoon
 * Added short disclaimer and copyright notice to beginning of file.
 *
 * Revision 1.7  1999/11/22 09:05:47  darkmoon
 * Added a little reminder about the _c(x) macro
 *
 * Revision 1.6  1999/11/20 10:00:01  darkmoon
 * Changed to v0.9 (build 9) - almost up to v1.0 release
 * Moved revision log to end of file
 * Added full list of keys that need to be supported
 * Added note about CON_COLS and CON_ROWS
 *
 * Revision 1.2  1999/11/08 00:50:27  darkmoon
 * Moved revision log to end of file.
 *
 * Revision 1.1.1.1  1999/11/06 05:04:39  darkmoon
 * Initial revision
 *
 * Revision 1.5  1999/05/18 08:35:02  darkmoon
 * Updated for Terminality 0.4 (build 8)
 *
 * Revision 1.4  1999/05/18 08:30:57  darkmoon
 * Updated header
 * Removed sizex()/sizey()
 * Removed tn_id[]
 *
 * Revision 1.3  1999/03/20 11:00:28  darkmoon
 * Updated version, now 0.3 - build 7
 *
 * Revision 1.2  1999/03/20 10:54:05  darkmoon
 * Fixed chtype for Win32
 *
 * Revision 1.1  1999/03/06 07:24:17  darkmoon
 * Renamed to Terminality
 * All files were brought up-to-standard (except the W32 stuff, hehe) for
 * the first release of Terminality (v0.2 build 6)
 *
 * Revision 1.22  1999/03/06 06:42:46  darkmoon
 * Added sizex() and sizey() functions.
 *
 * Revision 1.21  1999/02/21 05:14:57  darkmoon
 * Now we have BLOCK_1 thru BLOCK_4
 *
 * Revision 1.20  1999/01/16 02:57:58  darkmoon
 * Added version info
 *
 * Revision 1.19  1999/01/16 02:54:31  darkmoon
 * That's KEY_DEL, not KEY_BACKSPACE
 *
 * Revision 1.18  1999/01/16 02:36:20  darkmoon
 * Added KEY_DEL capability
 *
 * Revision 1.17  1999/01/11 04:09:00  darkmoon
 * Added [high/low/norm/reverse]video functions.
 *
 * Revision 1.16  1998/12/09 07:01:23  darkmoon
 * Compatibility tweaks
 *
 * Revision 1.15  1998/12/09 06:27:19  darkmoon
 * Changes to fit Win32 version.
 *
 * Revision 1.14  1998/12/07 10:01:56  darkmoon
 * Added console width/height stuff.
 *
 * Revision 1.13  1998/12/07 09:53:42  darkmoon
 * Fixed sentinel
 * Added C++ compatibility (extern "C" {...})
 *
 * Revision 1.12  1998/12/04 08:29:03  darkmoon
 * Fixed writech() problem
 *
 * Revision 1.11  1998/12/02 10:43:36  darkmoon
 * Added update()
 *
 * Revision 1.10  1998/12/02 10:13:45  darkmoon
 * Added writech()
 *
 * Revision 1.9  1998/12/02 10:08:26  darkmoon
 * Added special characters
 *
 * Revision 1.8  1998/12/02 09:58:55  darkmoon
 * Added gotoxy()
 *
 * Revision 1.7  1998/12/02 09:54:23  darkmoon
 * Added readkey(), printw(), delay()
 *
 * Revision 1.6  1998/12/02 09:40:13  darkmoon
 * Changed COLOR[][] definition from short to int
 *
 * Revision 1.5  1998/12/02 09:20:16  darkmoon
 * Fixed color/colour
 *
 * Revision 1.4  1998/12/02 09:18:33  darkmoon
 * Added color enum
 * Added general color support
 *
 * Revision 1.3  1998/12/01 10:48:20  darkmoon
 * Added (incomplete) list of functions
 *
 * Revision 1.2  1998/12/01 10:32:51  darkmoon
 * Added platform-dependent include for NCURSES
 *
 * Revision 1.1.1.1  1998/12/01 10:15:51  darkmoon
 * Setting up
 * 
 */

